import { getChartCategory, getEntityProperty, getPropertyHierarchy, getPropertyCaption, mergeOptions } from '@metad/ocap-core';
import { isNil } from 'lodash-es';
import { getCoordinateSystem, measuresToSeriesComponents, serializeSeriesComponent } from './cartesian';
import { coordinates, gatherCoordinates } from './coordinates';
import { getEChartsTooltip } from './tooltip';
export function pie(data, chartAnnotation, entityType, settings, options) {
    if (data.data?.length > (settings?.maximumLimit ?? 100)) {
        throw new Error(`Too much data, length ${data.data?.length}`);
    }
    const pieCoordinates = coordinates(data, chartAnnotation, entityType, settings, options, 'pie', pieCoordinate);
    return gatherCoordinates(pieCoordinates, 'pie', options);
}
/**
 * 单个笛卡尔坐标系, 对应一个 grid 内的组件
 *
 * @param data 数据数组
 * @param chartAnnotation 图形注解
 * @param entityType 实体类型
 * @param settings 图形设置
 * @param options 图形属性
 * @returns
 */
export function pieCoordinate(data, chartAnnotation, entityType, settings, options) {
    const category = getChartCategory(chartAnnotation);
    const categoryProperty = getEntityProperty(entityType, category);
    let datasets = [];
    const seriesComponents = measuresToSeriesComponents(chartAnnotation.measures, data, entityType, settings);
    datasets = [
        {
            dataset: {
                source: data,
                // 非矩阵数据集应该不用定义 dimensions , 因为会直接使用字段名作为 encode
                // dimensions: [
                //   ...chartAnnotation.dimensions.map(getPropertyName),
                //   ...chartAnnotation.measures.map(getPropertyMeasure)
                // ]
            },
            seriesComponents,
            tooltip: getEChartsTooltip(options?.tooltip, categoryProperty, chartAnnotation.measures.map((measure) => ({
                measure,
                property: getEntityProperty(entityType, measure)
            })), seriesComponents, settings.locale)
        }
    ];
    const { categoryAxis, valueAxis } = getCoordinateSystem(chartAnnotation, entityType, data, options, settings.locale);
    const gridOptions = {
        type: 'Pie',
        name: '',
        grid: mergeOptions({
            top: 0,
            right: 0,
            bottom: 0,
            left: 0,
            containLabel: true
        }, options?.grid),
        visualMap: [],
        datasets: [],
        tooltip: [],
        legend: [],
    };
    datasets.forEach(({ dataset, seriesComponents, tooltip }) => {
        gridOptions.datasets.push({
            dataset,
            series: seriesComponents.map((seriesComponent) => {
                const { series, visualMaps } = serializeSeriesComponent(dataset, seriesComponent, entityType, valueAxis, settings, options);
                gridOptions.visualMap.push(...visualMaps);
                /**
                 * 堆积系列情况下不适用 encode, 使用的应该是 seriesLayoutBy: "row" name: "[Canada]" 与 dataset 中的行数据进行的对应
                 */
                if (isNil(series.seriesLayoutBy)) {
                    series.encode = {
                        itemId: getPropertyHierarchy(category),
                        itemName: getPropertyCaption(categoryProperty),
                        value: seriesComponent.measure,
                        tooltip: seriesComponent.tooltip
                    };
                }
                if (chartAnnotation.chartType.variant === 'Doughnut') {
                    series.radius = ['40%', '80%'];
                    series.itemStyle = {
                        borderRadius: 5
                    };
                }
                if (chartAnnotation.chartType.variant === 'Nightingale') {
                    series.roseType = 'area';
                    series.radius = ['20%', '80%'];
                    series.itemStyle = {
                        borderRadius: 5
                    };
                }
                series.tooltip = tooltip;
                mergeOptions(series, options?.seriesStyle);
                return series;
            })
        });
        gridOptions.tooltip.push(tooltip);
    });
    // legend
    if (options?.legend) {
        gridOptions.legend.push(options.legend);
    }
    return gridOptions;
}
//# sourceMappingURL=pie.js.map