import { FilteringLogic, getChartCategory, getChartSeries, getDimensionLabel, getEntityHierarchy, getPropertyCaption, getPropertyHierarchy, isChartMapType, mergeOptions, SmartChartEngine } from '@metad/ocap-core';
import { isEqual, isEmpty, isArray, isNil } from 'lodash-es';
import { BehaviorSubject, catchError, combineLatest, concatMap, debounceTime, distinctUntilChanged, EMPTY, filter, from, map, Observable, of, shareReplay, switchMap, tap, withLatestFrom } from 'rxjs';
import { bar } from './bar';
import { bar3d } from './bar3d';
import { heatmap } from './heatmap';
import { sankey, sunburst, tree, treemap } from './hierarchy';
import { line } from './line';
import { line3d } from './line3d';
import { mapChart, mapChartAnnotation } from './maps/map';
import { Funnel } from './measures';
import { pie } from './pie';
import { scatter } from './scatter';
import { scatter3d } from './scatter3d';
import { themeRiver } from './themeRiver';
export class SmartEChartEngine extends SmartChartEngine {
    constructor() {
        super({});
        this.echartsInstance$ = new BehaviorSubject(null);
        this.error$ = new BehaviorSubject(null);
        this.echartsOptions$ = combineLatest([
            this.data$.pipe(filter((value) => !!value)
            // tap((value) => console.log(`echarts data changed:`, value))
            ),
            this.chartAnnotation$.pipe(filter((chartAnnotation) => !isEmpty(chartAnnotation?.dimensions) || !isEmpty(chartAnnotation?.measures)), concatMap((chartAnnotation) => {
                if (isChartMapType(chartAnnotation.chartType)) {
                    return from(mapChartAnnotation(chartAnnotation));
                }
                return of(chartAnnotation);
            })
            // tap((value) => console.log(`echarts chartAnnotation changed:`, value))
            ),
            this.settings$.pipe(distinctUntilChanged(isEqual)),
            this.options$.pipe(distinctUntilChanged(isEqual))
        ]).pipe(debounceTime(100), withLatestFrom(this.entityType$), map(([[data, chartAnnotation, settings, chartOptions], entityType]) => {
            try {
                settings = { ...(settings ?? {}) };
                settings.locale = settings.locale ?? 'en';
                // Custom Logic
                if (settings.customLogic) {
                    let customLogic;
                    try {
                        customLogic = new Function('data', 'chartAnnotation', 'entityType', 'locale', 'chartsInstance', settings.customLogic);
                    }
                    catch (error) {
                        console.error(error);
                        this.error$.next(error.message);
                        customLogic = null;
                    }
                    if (customLogic) {
                        return customLogic(data, chartAnnotation, entityType, settings.locale, this.echarts);
                    }
                }
                const type = chartAnnotation.chartType?.type;
                let options = null;
                if (type === 'Bar') {
                    options = bar(data, chartAnnotation, entityType, settings, chartOptions);
                }
                if (type === 'Line') {
                    options = line(data, chartAnnotation, entityType, settings, chartOptions);
                }
                if (type === 'Pie') {
                    options = pie(data, chartAnnotation, entityType, settings, chartOptions);
                }
                if (type === 'Scatter' || type === 'EffectScatter') {
                    options = scatter(data, chartAnnotation, entityType, settings, chartOptions);
                }
                if (type === 'Heatmap') {
                    options = heatmap(data, chartAnnotation, entityType, settings, chartOptions);
                }
                if (type === 'Bar3D') {
                    options = bar3d(data, chartAnnotation, entityType, settings, chartOptions);
                }
                if (type === 'Scatter3D') {
                    options = scatter3d(data, chartAnnotation, entityType);
                }
                if (type === 'Line3D') {
                    options = line3d(data, chartAnnotation, entityType);
                }
                if (type === 'Tree') {
                    options = tree(data, chartAnnotation, entityType, settings, chartOptions);
                }
                if (type === 'Treemap') {
                    options = treemap(data, chartAnnotation, entityType, settings, chartOptions);
                }
                if (type === 'Sunburst') {
                    options = sunburst(data, chartAnnotation, entityType, settings, chartOptions);
                }
                if (type === 'Sankey') {
                    options = sankey(data, chartAnnotation, entityType, settings, chartOptions);
                }
                if (type === 'Funnel') {
                    options = Funnel(data, chartAnnotation, entityType, settings, chartOptions);
                }
                if (type === 'ThemeRiver') {
                    options = themeRiver(data, chartAnnotation, entityType, settings, chartOptions);
                }
                if (type === 'GeoMap') {
                    options = mapChart(data, chartAnnotation, entityType, settings, chartOptions);
                }
                if (!options) {
                    throw new Error(`Unimplements type '${type}'`);
                }
                if (!isEmpty(chartOptions?.colors?.color)) {
                    options.color = chartOptions.colors.color;
                }
                if (chartOptions?.aria) {
                    options.aria = mergeOptions(options.aria, chartOptions.aria);
                }
                return {
                    options
                };
            }
            catch (err) {
                console.error(err);
                this.error$.next(err.message);
                return {
                    error: err.message
                };
            }
        }), tap((options) => console.log(options)), catchError((err) => {
            console.error(err);
            this.error$.next(err);
            return EMPTY;
        }), shareReplay(1));
        // echarts mouse events
        this.chartClick$ = this.createLazyEvent('click').pipe(withLatestFrom(this.chartAnnotation$, this.entityType$), map(([event, chartAnnotation, entityType]) => {
            const dimension = getChartCategory(chartAnnotation);
            const chartSeries = getChartSeries(chartAnnotation);
            const property = getEntityHierarchy(entityType, dimension);
            const hierarchyName = property.name;
            let item = event.data?.item || event.data;
            let ftr;
            if (chartSeries) {
                if (isArray(item)) {
                    item = item[item.length - 1];
                }
                const value = isArray(item) ? 'XXX' : item[getPropertyHierarchy(chartSeries)]; // event.dimensionNames[event.encode[AxisEnum[this.valueAxis]][0]]
                const sFilter = {
                    dimension: chartSeries,
                    members: [
                        {
                            value
                            // label: this.legendTexts?.get(value)
                        }
                    ]
                };
                ftr = {
                    filteringLogic: FilteringLogic.And,
                    children: [
                        // TODO path -> dimension
                        {
                            dimension,
                            members: [
                                {
                                    value: item[hierarchyName],
                                    label: item[getPropertyCaption(property)]
                                }
                            ]
                        },
                        sFilter
                    ]
                };
            }
            else if (item) {
                // filter 中 path 优先使用 hierarchy
                const value = item[hierarchyName] ?? item.rawValue?.[hierarchyName];
                const label = item[getPropertyCaption(property)] ?? item.rawValue?.[getPropertyCaption(property)];
                ftr = {
                    dimension,
                    members: [
                        {
                            value,
                            label
                        }
                    ]
                };
            }
            return { ...event, event: event.event?.event, filter: ftr };
        }));
        /**
         * https://echarts.apache.org/en/api.html#events.selectchanged
         */
        this.selectChanged$ = this.createLazyEvent('selectchanged').pipe(withLatestFrom(this.echartsOptions$, this.chartAnnotation$, this.entityType$), debounceTime(100), withLatestFrom(this.chartClick$), map(([[event, echartsOptions, chartAnnotation, entityType], chartClick]) => {
            console.warn(event, chartClick, echartsOptions);
            const category = getChartCategory(chartAnnotation);
            const categoryProperty = getEntityHierarchy(entityType, category);
            const label = getDimensionLabel(entityType, category);
            const chartSeries = getChartSeries(chartAnnotation);
            const slicers = event.selected
                .filter(({ dataType }) => !(dataType && dataType !== 'node'))
                .map(({ seriesIndex, dataIndex }) => {
                const series = echartsOptions.options.series[seriesIndex];
                const datasetIndex = series.datasetIndex;
                if (isNil(datasetIndex)) {
                    // Not use dataset, series inline data
                    return {
                        dimension: category,
                        members: dataIndex.map((index) => ({
                            value: series.data[index][category.dimension] ?? series.data[index].rawValue?.[category.dimension],
                            label: series.data[index][label] ?? series.data[index].rawValue?.[label]
                        }))
                    };
                }
                const dataset = echartsOptions.options.dataset[datasetIndex];
                if (dataset.transform) {
                    return chartClick.filter;
                }
                if (chartSeries) {
                    // return chartClick.filter
                    // 逻辑与有 Series 维度下生成逻辑一致, 即数据使用的是数组矩阵 DataSet 的形式
                    return {
                        filteringLogic: FilteringLogic.And,
                        children: [
                            {
                                dimension: category,
                                members: dataIndex.map((index) => {
                                    const value = dataset.source[0][index + 1];
                                    return {
                                        value,
                                        label: dataset.categories?.find((item) => item.value === value)?.label
                                    };
                                })
                            },
                            {
                                dimension: chartSeries,
                                members: [
                                    {
                                        value: series.name,
                                        label: series.caption
                                    }
                                ]
                            }
                        ]
                    };
                }
                return {
                    dimension: category,
                    members: dataIndex.map((index) => ({
                        value: dataset.source[index][category.dimension],
                        label: dataset.source[index][label]
                    }))
                };
            });
            return { event: chartClick.event, slicers };
        }));
    }
    get echarts() {
        return this.echartsInstance$.value;
    }
    set echarts(value) {
        this.echartsInstance$.next(value);
    }
    selectChartOptions() {
        return this.echartsOptions$;
    }
    createLazyEvent(eventName) {
        return this.echartsInstance$.pipe(filter((c) => !!c), switchMap((chart) => new Observable((observer) => {
            chart.on(eventName, (params) => {
                observer.next(params);
            });
            return () => {
                if (!chart?.isDisposed()) {
                    chart.off(eventName);
                }
            };
        })));
    }
}
//# sourceMappingURL=smart-echarts.js.map